var _ = require('lodash');

exports.makeCanvas = function(spec) {

  spec = _.defaults(spec || {}, {
    width: 750,
    height: 500
  });

  let canvas = document.createElement('canvas');

  canvas.width = spec.width;
  canvas.height = spec.height;

  let context = canvas.getContext('2d');
  context.imageSmoothingEnabled = false;
  context.webkitImageSmoothingEnabled = false;
  context.mozImageSmoothingEnabled = false;

  return canvas;
};

exports.makeStretchyCanvas = function(spec) {

  spec = _.defaults(spec || {}, {
    width: 750,
    height: 500
  });

  let canvas = exports.makeCanvas();

  canvas.style.display = 'block';
  canvas.style.margin = 'auto';

  let aspect = spec.width / spec.height;

  function update() {
    var bounds = {height: window.innerHeight, width: window.innerWidth};

    if (bounds.width / bounds.height < aspect) {
      canvas.style.width = bounds.width + 'px';
      canvas.style.height = Math.floor(bounds.width / aspect) + 'px';
    }
    else {
      canvas.style.height = bounds.height + 'px';
      canvas.style.width = Math.floor(bounds.height * aspect) + 'px';
    }

    canvas.height = spec.height;
    canvas.width = spec.width;
  }

  update();

  canvas.update = update;

  return canvas;
};

exports.clear = function(context, col) {
  if (col) {
    context.fillStyle = col;
    context.fillRect(0, 0, context.canvas.width, context.canvas.height);
  }
  else
    context.clearRect(0, 0, context.canvas.width, context.canvas.height);
};
