var _ = require('lodash');
var assert = require('assert');

module.exports = function(spec) {

  let ng = {};

  let systems = [];
  let byName = {};

  let nextId = 1;

  function syscall(name) {
    var args = _.toArray(arguments).slice(1);
    systems.forEach(function(sys) {
      var fn = sys[name];
      if (fn)
        fn.apply(null, args);
    });
  }

  ng.makeID = function() {
    return nextId++;
  };

  ng.update = function(dt) {
    syscall('update', dt);
  };

  ng.frame = function(dt) {
    syscall('frame', dt);
  };

  ng.addSystem = function(_name, _sys) {
    let args = _.toArray(arguments);
    let sys = args.pop();
    let name = args.shift();

    systems.push(sys);

    if (name) {
      assert(!byName[name]);

      sys.name = name;
      byName[name] = sys;
    }
  };

  ng.systems = byName;

  function init(name, mod) {
    ng.addSystem(name, mod(ng, spec[name]));
  }

  init('transform', require('ace/systems/transform'));
  init('render', require('ace/systems/render'));
  init('physics', require('ace/systems/physics'));
  init('script', require('ace/systems/script'));
  init('reaper', require('ace/systems/reaper'));

  return ng;

};
