var cell = require('frp').cell;
var R = require('mxd').Rect;
var V = require('mxd').Vector;

var T = require('ace/transform');

exports.load = function(url) {
  var r = cell();
  var img = new Image();
  img.src = url;
  img.onload = function() {
    r.set(img);
  };
  return r;
};

exports.image = function(image, origin) {

  origin = origin || [image.width / 2, image.height / 2];

  return {
    bounds: function(view) {
      return R.fromSize(V.invert(origin), [image.width, image.height]);
    },
    draw: function(view) {
      view.withContext(function(c) {
        c.drawImage(
          image,
          -origin[0],
          -origin[1]
        );
      });
    }
  };
};

exports.tile = function(gx) {
  return {
    draw: function(view) {

      var b = gx.bounds(view);

      var vtr = view.getTransform();

      var dPos = V.invert(V.div(vtr.pos, vtr.scale));

      var viewSize = view.getViewport();
      var numCopies = V.ceil(V.div(viewSize, b.size));

      V.iterect([0, 0], numCopies, function(gridPos) {

        var dStartPos = V.add(
          dPos,
          V.sub(
            V.add(V.mod(vtr.pos, b.size), V.mul(b.size, gridPos)),
            b.size
          )
        );

        var d = T.pos(dStartPos);
        var w = view.transform(d);

        gx.draw(w);

      });

    }
  };
};

exports.parallax = function(gx, factor) {
  return {
    draw: function(view) {
      var vtr = view.getTransform();
      var w = view.transform(T.pos(
        V.mul(V.invert(vtr.pos), V.div(factor, vtr.scale))
      ));
      gx.draw(w);
    }
  };
};


exports.animation = function(gxs, duration) {

  let nFrames = gxs.length;
  let frameDur = duration / nFrames;

  return {
    bounds: function(view) { return gxs[0].bounds(view); },
    draw: function(view) {

      let f = view.time % duration;
      let i = Math.floor(f / frameDur);

      return gxs[i].draw(view);
    }
  };
};
