var T = require('../transform');
var V = require('mxd').Vector;

var p2 = require('p2');
var _ = require('lodash');

var debugDraw = require('ace/debugDraw.js');

module.exports = function(ng) {

  var world = new p2.World({gravity: [0, 500]});

  var sys = {};

  sys.world = world;

  var items = {};

  sys.items = items;

  sys.setBody = function(id, body) {
    world.addBody(body);
    items[id] = {
      body: body, 
      force: [0, 0]
    };
  };

  sys.getBody = function(id) {
    return items[id];
  };

  sys.update = function(dt) {

    ng.systems.reaper.cleanup(items, function(pnode, id) {
      if (pnode.body)
        world.removeBody(pnode.body);
    });

    world.step(dt);

    _.forEach(items, function(item, id) {

      var body = item.body;
      body.force = V.copy(item.force);

      var pos = [body.position[0], body.position[1]];

      var tr = ng.systems.transform.getTransform(id);
      var tr1 = T.copy(tr, {pos: pos, rot: body.angle });
      ng.systems.transform.setTransform(id, tr1);

    });
  };

  sys.makeDebugGraphic = function() {
    return {
      draw: function(view) {
        view.withContext(function(context) {
          debugDraw.world(context, world);
        });
      }
    };
  };

  return sys;
};
