let _ = require('lodash');
let canvasUtils = require('ace/canvasUtils');
let T = require('ace/transform');
let V = require('mxd/Vector');
let makeView = require('ace/view');

module.exports = function(ng, spec) {

  let sys = {};

  let canvas = canvasUtils.makeCanvas(spec);
  document.getElementById('game').appendChild(canvas);

  let camId = null;

  sys.setCamera = function(id) {
    camId = id;
  };

  sys.nodes = {};

  var time = 0;
  var flash = false;

  sys.frame = function(dt) {

    flash = !flash;

    ng.systems.reaper.cleanup(sys.nodes);

    time += dt;

    let c = canvas.getContext('2d');

    canvasUtils.clear(c);

    var transform = ng.systems.transform;

    var view = makeView(c, T.identity, time);

    if (camId !== null) {
      var camPos = transform.getPos(camId);
      var offset = V.sub(camPos, [canvas.width/2, canvas.height/2]);
      view = view.transform(T.pos(V.invert(offset)));
    }

    _.forEach(sys.nodes, function(node, id) {

      var gx = sys.getGraphic(id);
      if (gx) {

        if (gx.flashing && !flash)
          return;

        if (gx.invisible)
          return;

        var tr = transform.getTransform(id);
        var nodeView = view.transform(tr);

        gx.draw(nodeView);

        var b = gx.bounds && gx.bounds(nodeView);
        if (0 && b) {
          nodeView.withContext(function(c) {
            c.beginPath();
            c.rect(b.left, b.top, b.width, b.height);
            c.stroke();
          });
        }
      }

    });

  };

  sys.setGraphic = function(id, gx) {
    sys.nodes[id] = gx;
  };

  sys.getGraphic = function(id) {
    return sys.nodes[id];
  };

  return sys;
};
