let _ = require('lodash');

let M = require('mxd/Matrix');
let V = require('mxd/Vector');
let R = require('mxd/Rect');

var T = module.exports = {};

T.prs = function(pos, rot, scale) {
  return Object.defineProperties({
    pos: pos,
    rot: rot,
    scale: scale
  }, {
    matrix: {get: _.once(function() {
      var m = M.mul(M.scaling(scale), M.rotation(rot));
      m[0][2] = pos[0];
      m[1][2] = pos[1];
      m[2] = [0, 0, 1];
      return m;
    })}
  });
};

T.copy = function(x, y) {
  var z = _.assign({}, x, y);
  return T.prs(z.pos, z.rot, z.scale);
};

T.identity = T.prs([0, 0], 0, [1, 1]);

T.pos = function(v) {
  return T.prs(v, 0, [1, 1]);
};

T.rot = function(a) {
  return T.prs([0, 0], a, [1, 1]);
};

T.scale = function(v) {
  return {
    pos: [0, 0],
    rot: 0,
    scale: v,
    matrix: M.scaling(v)
  };
};

T.mul = function(a, b) {
  return T.prs(
    T.apply(b, a.pos),
    a.rot + b.rot,
    V.mul(a.scale, b.scale)
  );
};

T.apply = function(t, v) {
  //return M.apply(t.matrix, [v[0], v[1], 1]).slice(0, 2);
  return V.add(t.pos, V.rotate(V.mul(v, t.scale), t.rot));
};

T.neg = function(t) {
  return T.prs(
    V.invert(t.pos),
    -t.rot,
    V.scaleInverse(t.scale, 1)
  );
};

T.translate = function(t, v) {
  return T.copy(t, {pos: V.add(t.pos, v)});
};

T.invert = function(t) {
  return T.prs(
    V.invert(V.mul(V.rotate(t.pos, -t.rot), t.scale)),
    -t.rot,
    V.scaleInverse(t.scale, 1)
  );
};

T.toContext = function(t) {
  var m = t.matrix;
  return [m[0][0], m[1][0], m[0][1], m[1][1], t.pos[0], t.pos[1]];
};

T.applyToContext = function(t, context) {
  context.transform.apply(context, T.toContext(t));
};
