var _ = require('lodash');
var graphics = require('ace/graphics');
var gfx = require('./gfx');
var items = require('./items');
var R = require('mxd').Rect;
var sound = require('./sound');
var music = require('./music');
var p2 = require('p2');
var T = require('ace/transform');

exports.going = false;

var spots = [150, 375, 600];
var snows;

exports.reveal = function(ng, gBg, gPlayer) {

  _.forEach(ng.systems.physics.items, function(pnode, id) {
    if (!pnode.body.rgaKeep)
      ng.systems.reaper.kill(id);
  });

  music.stop();

  exports.going = true;
  ng.systems.render.setGraphic(gBg, graphics.image(gfx.reveal, [0, 0]));

  snows = spots.map(function(x) {
    return makeSnow(ng, [x, 380]);
  });


  items.floor = 380;
  items.makePlatform(ng, R.fromSize([0, items.floor], [750, 60]));

  var snowsit = ng.makeID();
  ng.systems.render.setGraphic(snowsit, graphics.image(gfx.snowsit, [0, 0]));


  var snowfa = ng.makeID();
  ng.systems.render.setGraphic(snowfa, graphics.image(gfx.snowfa));
  ng.systems.transform.setPos(snowfa, [375, 500 - 102/2]);

  ng.systems.physics.getBody(gPlayer).body.position = [375, 220];
  ng.systems.physics.getBody(gPlayer).body.velocity = [0, 0];

  setTimeout(function() {
    ng.systems.reaper.kill(snowsit);
    sound.play('bigjump');
    makeSnowman(ng);
  }, 4000);

};

function makeSnow(ng, pos) {
  var id = ng.makeID();
  ng.systems.render.setGraphic(id, graphics.image(gfx.snow2));
  ng.systems.render.nodes[id].invisible = true;

  var body = new p2.Body({mass: 0, position: pos});
  var shape = new p2.Rectangle(120, 30);
  shape.sensor = true;
  body.addShape(shape);
  body.data = id;
  body.rgaSnow = 1;
  ng.systems.physics.setBody(id, body);

  return id;
}

function makeSnowman(ng) {
  var id = ng.makeID();
  ng.systems.render.setGraphic(id, graphics.image(gfx.snowman));
  ng.systems.transform.setPos(id, [600, 270]);

  var body = new p2.Body({mass: 0, position: [600, 270]});
  var shape = new p2.Rectangle(100, 250);
  body.addShape(shape);
  ng.systems.physics.setBody(id, body);

  var hp = 11;

  function killit() {
    ng.systems.reaper.kill(id);

    setTimeout(function() {
      var end = ng.makeID();
      var gg = graphics.image(gfx.end, [0, 0]);
      gg.ord = -1;
      ng.systems.render.setGraphic(end, gg);
    }, 1000);
  }

  var invin = false;
  ng.systems.physics.world.on('beginContact', function(e) {
    if (invin)
      return;
    if (e.shapeA === shape || e.shapeB === shape) {
      let otherBody = (e.shapeA === shape) ? e.bodyB : e.bodyA;
      if (otherBody.rgaTag === 'crate' || otherBody.rgaTag == 'snowball') {

        if (otherBody.rgaTag == 'snowball')
          ng.systems.reaper.kill(otherBody.data);

        sound.play('hurt');
        ng.systems.render.nodes[id].flashing = true;
        invin = true;
        setTimeout(function() {
          invin = false;
          (ng.systems.render.nodes[id] || {}).flashing = false;
        }, 500);

        hp -= 1;
        if (hp === 0) {
          killit();
        }

      }
    }
  });

  var spoti = 2;
  var dir = -1;

  var moving = false;
  var mstart = 0;
  var t = 0;
  var moveDur = 1.5;

  function move() {
    var nspoti = spoti + dir;
    if (0 <= nspoti && nspoti < spots.length) {

      spoti = nspoti;
      moving = true;
      mstart = t;
    }
    else {
      dir = -dir;
      var tr = ng.systems.transform.getTransform(id);
      ng.systems.transform.setTransform(id, T.copy(tr, {scale: [-dir, 1]}));
      setTimeout(function() { move(); }, 1000);
    }
  }


  ng.addSystem({
    update: function(dt) {
      if (hp <= 0) return;
      t += dt;

      if (moving) {
        var fromX = spots[spoti - dir];
        var toX = spots[spoti];
        var f = (t - mstart) / moveDur;
        if (f >= 1) {
          f = 1;
          moving = false;
          setTimeout(function() { move(); }, 1000);
          var gn = ng.systems.render.nodes[snows[spoti]];
          gn.invisible = false;

        }
        var x = fromX + f * (toX - fromX);

        var y = 270 - Math.sin(f * Math.PI) * 100;

        var b = ng.systems.physics.getBody(id);
        if (b)
          b.body.position = [x, y];
      }
    }
  });

  setTimeout(move, 2000);

}

