var _ = require('lodash');
var p2 = require('p2');
var graphics = require('ace/graphics');
var gfx = require('./gfx');
var materials = require('./materials');
var V = require('mxd').Vector;

exports.floor = 440;

exports.system = function(engine) {

  var transform = engine.systems.transform;
  var physics = engine.systems.physics;

  var held = {};

  var sys = {};

  sys.holding = function(holder) {
    return (
      _.pairs(held).
      filter(function(x) { return x[1] === holder; })
      .map(function(x) { return x[0]; })
    );
  };

  sys.pickup = function(holder, item) {
    held[item] = holder;

    var body = physics.getBody(item);
    body.oldMass = body.body.mass;
    body.body.mass = 0;
    body.body.shapes[0].sensor = true;
  };

  sys.drop = function(item) {
    delete held[item];

    var tr = transform.getTransform(item);

    var body = physics.getBody(item);
    body.body.mass = body.oldMass;
    body.body.velocity = [0, 0];
    body.body.setZeroForce();
    body.body.position = tr.pos;
    body.body.shapes[0].sensor = false;
  };


  sys.update = function(dt) {
    _.forEach(held, function(holder, item) {
      var trh = transform.getPos(holder);
      var tri = transform.getPos(item);
      transform.setPos(item, V.add(trh, [0, -50]));
    });
  };

  return sys;
};

exports.makeBox = function(engine, pos, straight) {
  let shadow = engine.makeID();
  let box = engine.makeID();

  engine.systems.render.setGraphic(shadow, graphics.image(gfx.shadow));
  engine.addSystem({
    frame: function() {
      var tr = engine.systems.transform.getPos(box);
      engine.systems.transform.setPos(shadow, [tr[0], exports.floor]);
    }
  });

  engine.systems.reaper.depend(shadow, box);

  engine.systems.transform.setPos(box, pos);

  var body = new p2.Body({ mass: 3, position: pos });
  if (!straight)
    body.angle = Math.random() * 2 * Math.PI;
  body.data = box;
  var shape = new p2.Rectangle(35, 35);
  shape.material = materials.env;
  body.addShape(shape);

  body.rgaTag = 'crate';

  engine.systems.physics.setBody(box, body);

  engine.systems.render.setGraphic(
    box,
    graphics.image(gfx.box)
  );

  return box;
};

exports.makePlatform = function(ng, rect, group) {
  var id = ng.makeID();

  var floor = new p2.Body({
    mass: 0,
    position: rect.centre
  });

  if(group)
    floor.rgaKeep = true;

  var floorShape = new p2.Rectangle(rect.width, rect.height);
  floorShape.collisionGroup = group || 1;
  floorShape.material = materials.env;
  floor.addShape(floorShape);

  ng.systems.physics.setBody(id, floor);

  return id;
};

exports.makeSnowball = function(engine, pos) {
  let shadow = engine.makeID();
  let ball = engine.makeID();

  engine.systems.render.setGraphic(shadow, graphics.image(gfx.shadow));
  engine.addSystem({
    frame: function() {
      var tr = engine.systems.transform.getPos(ball);
      engine.systems.transform.setPos(shadow, [tr[0], exports.floor]);
    }
  });

  engine.systems.reaper.depend(shadow, ball);

  engine.systems.transform.setPos(ball, pos);

  var body = new p2.Body({ mass: 1, position: pos });
  body.data = ball;
  var shape = new p2.Circle(15);
  shape.material = materials.env;
  body.addShape(shape);

  body.rgaTag = 'snowball';

  engine.systems.physics.setBody(ball, body);

  engine.systems.render.setGraphic(
    ball,
    graphics.image(gfx.snowball)
  );

  return ball;
};

exports.makeApple = function(engine, pos) {
  let shadow = engine.makeID();
  let apple = engine.makeID();

  engine.systems.render.setGraphic(shadow, graphics.image(gfx.shadow));
  engine.addSystem({
    frame: function() {
      var tr = engine.systems.transform.getPos(apple);
      engine.systems.transform.setPos(shadow, [tr[0], exports.floor]);
    }
  });

  engine.systems.reaper.depend(shadow, apple);

  engine.systems.transform.setPos(apple, pos);

  var body = new p2.Body({ mass: 1, position: pos });
  body.data = apple;
  var shape = new p2.Circle(15);
  shape.material = materials.env;
  body.addShape(shape);

  body.rgaTag = 'apple';

  engine.systems.physics.setBody(apple, body);

  engine.systems.render.setGraphic(
    apple,
    graphics.image(gfx.apple)
  );

  return apple;
};
