var midifile = require('midifile');

function getBin(url, cb) {
  var rq = new XMLHttpRequest();
  rq.open('GET', url);
  rq.responseType = 'arraybuffer';
  rq.onload = function() {
    if (rq.response)
      cb(rq.response);
  };
  rq.send(null);
}

var nowPlaying;
var gGain;

exports.stop = function() {
  if (gGain) {
    gGain.disconnect();
    nowPlaying = 0;
  }
};


var id = 0;
exports.play = function(filename) {

  var thisId = ++id;
  nowPlaying = thisId;

  if (1 && window.location.href.indexOf('foon') == -1)
    return;

  getBin(filename, function(file) {

    if (1) {

      var midi = new midifile(file);

      var events = midi.getMidiEvents();

      var endAt;
      var meta = midi.getEvents();
      meta.forEach(function(event) {
        if (event.subtype == 47)
          endAt = event.playTime;
      });

      events.forEach(function(event) {
        if(event.type===midifile.EVENT_META&&event.text) {
              console.log('Text meta: '+event.text);
          }
      });
    }

    window.AudioContext = window.AudioContext || window.webkitAudioContext;

    var context = new AudioContext();
    window.context = context;

    var A = .01;
    var D = .1;

    var S = .25;
    var R = .5;

    var gain = gGain = context.createGain();
    gain.connect(context.destination);
    gain.gain.value = .3;

    var voices = [];
    for (var i=0; i<16; i++) {
      var v = {
        osc: context.createOscillator(),
        filt: context.createBiquadFilter(),
        gain: context.createGain()
      };

      v.osc.type = 'sawtooth';
      v.osc.connect(v.filt);

      v.filt.type = 'lowpass';
      v.filt.frequency.value = 800;
      v.filt.connect(v.gain);

      v.gain.connect(gain);

      v.osc.frequency.setValueAtTime(220, 0);
      v.gain.gain.value = 0;
      v.osc.start(0);
      voices.push(v);
    }

    function midiToFreq(i) {
      return 440 * Math.pow(2, (i - 69) / 12);
    }

    var timestep = 5;
    var time = 0;


    function process() {

      var now = 0;
      var end = now + 150000;

      var voicetab = {};
      var voiceno = 0;

      var ptr = 0;

      while(1) {
        var event = events[ptr++];
        if (!event)
          break;

        if (event.playTime < now) {
          continue;
        }

        var eTime = context.currentTime + (event.playTime - now) / 1000;

        if (event.subtype != 8 && event.subtype != 9) {
          continue;
        }

        var vi;

        if (event.subtype == 9) {
          vi = voiceno = (voiceno + 1) % voices.length;
        }

        if (event.subtype == 8) {
          vi = voicetab[event.param1];
          if (vi === undefined) {
            continue;
          }
        }

        var osc = voices[vi].osc, gain = voices[vi].gain;


        if (event.subtype == 8) {
          gain.gain.cancelScheduledValues(eTime);
          gain.gain.linearRampToValueAtTime(S, eTime);
          gain.gain.linearRampToValueAtTime(0, eTime + R);
        }

        if (event.subtype == 9) {
          voicetab[event.param1] = vi;
          var freq = midiToFreq(event.param1);
          osc.frequency.setValueAtTime(freq, eTime);
          gain.gain.cancelScheduledValues(eTime);
          gain.gain.setValueAtTime(0, eTime);
          gain.gain.linearRampToValueAtTime(0, eTime);
          gain.gain.linearRampToValueAtTime(1, eTime + A);
          gain.gain.linearRampToValueAtTime(S, eTime + A + D);
          gain.gain.setValueAtTime(.5, eTime);
        }

        if (event.playTime >= end)
          break;
      }

      setTimeout(function() {
        if (nowPlaying === thisId)
          process();
      }, endAt);
    }

    process();




  });
};
