var _ = require('lodash');
var p2 = require('p2');
var gfx = require('./gfx');
var materials = require('./materials');
var graphics = require('ace/graphics');
var proutines = require('proutines');
var T = require('ace/transform');
var V = require('mxd').Vector;

var items = require('./items');


function makeVehicle(ng, pos, vel, sx, body, image) {
  var physics = ng.systems.physics;
  var transform = ng.systems.transform;

  let id = ng.makeID();

  // Pos
  transform.setTransform(id, {
    pos: pos,
    scale: [sx, 1]
  });

  // Gfx
  ng.systems.render.setGraphic(id, graphics.image(image));

  // Physics
  physics.setBody(id, body);

  ng.systems.drive.set(id, vel);

  ng.systems.script.set(id, proutines.series([
    proutines.waitUntil(function() {
      var x = transform.getPos(id)[0];
      return x < -250 || x > 1000;
    }),
    proutines.action(function() {
      ng.systems.reaper.kill(id);
    })
  ])());

  return id;

}


exports.pusher = function(ng, pos, vel, sx) {
  var body = new p2.Body({ type: p2.Body.KINEMATIC, position: pos });

  var shape = new p2.Rectangle(90, 100);
  shape.material = materials.env;
  body.addShape(shape, [-45 * sx, 0]);

  shape = new p2.Rectangle(90, 50);
  shape.material = materials.env;
  body.addShape(shape, [45 * sx, 25]);

  return makeVehicle(ng, pos, vel, sx, body, gfx.pusher);
};

exports.minelayer = function(ng, pos, vel, sx) {
  var body = new p2.Body({ position: pos, type: p2.Body.KINEMATIC });

  var shape = new p2.Rectangle(180, 80);
  shape.material = materials.env;
  body.addShape(shape, [0, 0]);


  return makeVehicle(ng, pos, vel, sx, body, gfx.minelayer);
};


exports.truck = function(ng, pos, vel, sx) {
  var body = new p2.Body({ position: pos, type: p2.Body.KINEMATIC });

  var shape = new p2.Rectangle(320, 90);
  shape.material = materials.env;
  body.addShape(shape, [0, 0]);

  return makeVehicle(ng, pos, vel, sx, body, gfx.truck);
};

exports.boxTruck = function(ng, pos, vel, sx) {
  var truck = exports.truck(ng, pos, vel, sx);

  var map = {};

  _.times(6, function(y) {
    _.times(6, function(x) {

      var ok;
      if (y === 0)
        ok = Math.random() > .5;
      else {
        ok = map[[y-1, x]] && map[[y-1, x+1]];
      }

      if (ok) {
        map[[y, x]] = 1;

        var bp = V.sum([
          pos,
          [-80, -80],
          [y * 19, 0],
          V.scale([x, -y], 40)
        ]);
        items.makeBox(ng, bp, true);
      }

    });
  });


  return truck;
};


exports.system = function(ng) {

  var physics = ng.systems.physics;

  var sys = {};

  var nodes = {};

  sys.set = function(id, drive) {
    nodes[id] = drive;
  };

  sys.update = function(dt) {
    _.forEach(nodes, function(drive, id) {
      var pnode = physics.getBody(id);
      if (pnode) {
        pnode.body.velocity = drive;
        //pnode.body.position = [
          //pnode.body.position[0] + drive[0] * dt,
          //pnode.body.position[1] + drive[1] * dt,
        //];
      }
    });
  };

  return sys;
};
